# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

MARKDOWN_STR = """
<style>
button[kind="primary"] {
    background: none!important;
    border: 0;
    padding: 20!important;
    color: grey !important;
    text-decoration: none;
    cursor: pointer;
    border: none !important;
    # float: right;
}
button[kind="primary"]:hover {
    text-decoration: none;
    color: white !important;
}
button[kind="primary"]:focus {
    outline: none !important;
    box-shadow: none !important;
    color:  !important;
}
</style>
"""
